/*
 * Initialisation code for the stage 2 payload.
 *
 * Copyright 2009 (C) Qi Hardware Inc.
 * Author: Wolfgang Spraul <wolfgang@sharism.cc>
 *
 * Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

.text
.extern c_main
.extern _tlb_entry
.extern _exc_entry
.extern _irq_entry
.extern _end_entries
.globl _start
.set noreorder

_start:
	/* Initialise the stack. */

	la $sp, 0x80080000

	/* Initialise the globals pointer. */

	lui $gp, %hi(_GLOBAL_OFFSET_TABLE_)
	ori $gp, $gp, %lo(_GLOBAL_OFFSET_TABLE_)

	/* Copy TLB handling instructions. */

	la $t0, _tlb_entry		/* start */
	li $t1, 0x80000000
	la $t2, _cache_entry		/* end */
_tlb_copy:
	lw $t3, 0($t0)
	addiu $t0, $t0, 4
	sw $t3, 0($t1)
	bne $t0, $t2, _tlb_copy
	addiu $t1, $t1, 4		/* executed in delay slot before branch */

	/* Copy cache handling instructions. */

	move $t0, $t2			/* start */
	li $t1, 0x80000100
	la $t2, _exc_entry		/* end */
_cache_copy:
	lw $t3, 0($t0)
	addiu $t0, $t0, 4
	sw $t3, 0($t1)
	bne $t0, $t2, _cache_copy
	addiu $t1, $t1, 4		/* executed in delay slot before branch */

	/* Copy exception handling instructions. */

	move $t0, $t2			/* start */
	li $t1, 0x80000180
	la $t2, _irq_entry		/* end */
_exc_copy:
	lw $t3, 0($t0)
	addiu $t0, $t0, 4
	sw $t3, 0($t1)
	bne $t0, $t2, _exc_copy
	addiu $t1, $t1, 4		/* executed in delay slot before branch */

	/* Copy IRQ handling instructions. */

	move $t0, $t2			/* start */
	li $t1, 0x80000200
	la $t2, _end_entries		/* end */
_irq_copy:
	lw $t3, 0($t0)
	addiu $t0, $t0, 4
	sw $t3, 0($t1)
	bne $t0, $t2, _irq_copy
	addiu $t1, $t1, 4		/* executed in delay slot before branch */

	/* Initialise interrupts. */

	mfc0 $t3, $12			/* CP0_STATUS */
	nop
	li $t4, 0xffbf00e0		/* BEV = 0 (not bootloader vectors); IM = disable all */
	and $t3, $t3, $t4		/* ... KSU = 0 (kernel mode); ERL = 0; EXL = 0; IE = 0 */
	li $t4, 0x0000ff04		/* IM = enable IM7..IM0; ERL = 1 (set by default) */
	or $t3, $t3, $t4
	mtc0 $t3, $12
	nop

	li $t3, 0x00800000		/* IV = 1 (use 0x80000200 for interrupts) */
	mtc0 $t3, $13			/* CP0_CAUSE */
	nop

	mtc0 $zero, $15			/* CP0_EBASE (should be zero anyway) */
	nop

	/* Start the program. */

	j c_main
	nop

.set reorder
